/*!
 *	@file
 *	@brief	メモリシステム共通インターフェイスクラス
 *
 *	@author kondo
 *	@date	2013/08/12
 */

/*!
 *	@defgroup	grp_MemorySystem	メモリシステム
 *
 *	メモリシステムは様々なメモリアロケーションタイプを統一形式で扱う為のインターフェースクラスです。<br>
 *	メモリの確保や解放などを全て同じ形式で取り扱えます。<br>
 *	コンストラクタやデストラクタなど new や delete のオーバーライドとして利用することで簡素ですっきりした独自のメモリ管理を実現する事ができます。
 *
 *	例えば new / delete は
 *	@code
 *	inline void *operator new(Size_T n) throw(){
 *		return OrgSys::Memory::IMemory::GetInst()->mallocAlignment(n);
 *	}
 *
 *	inline void operator delete(void* p) throw() {
 *		OrgSys::Memory::IMemory::GetInst()->free(p);
 *	}
 *	@endcode
 *
 *	を用意する事で、通常のnew / deleteを差し替える事ができます。
 *
 *	IMemory は各種メモリシステムが共通で持つメソッドをまとめたインターフェイスクラスです。<br>
 *	IMemory はその実装クラスによって特徴が大きく変化します。<br>
 *
 *	@b メモリープールの種類<br>
 *	<table border=1 cellspacing=0>
 *	<tr><th>クラス名</th>				<th>名称</th>							<th>用途</th>	<th>速度</th>	<th>断片化耐性</th>
 *	<tr><td>dlMallocMemory</td>	<td>dlmallocメモリ</td>			<td>通常プログラム、データ用</td>	<td>普通</td>	<td>ややしやすい</td>	</tr>
 *	<tr><td>externalMcbMemory</td>	<td>外部メモリ</td>			<td>CPUが直接アクセスできないメモリの管理</td>	<td>低速</td>	<td>普通</td>	</tr>
 *	<tr><td>fixedSizeMemory</td>	<td>固定サイズ割当メモリ</td>	<td>固定サイズの割当て</td>	<td>超高速</td>	<td>断片化しない</td>	</tr>
 *	</table>
 *
 *	@b メモリシステムの作成
 *
 *	IMemory クラスを継承して各種メモリクラスに組み込む事で作成されます。
 *
 *	@b メモリシステムからのメモリ確保/解放
 *
 *	メモリからの確保は IMemory::mallocAlignment を使用します。
 *	解放に関しては IMemory::free を使用します。
 *
 *	ここで注意しないといけないのは、確保と解放は同種のメモリシステム上で行う必要があります。<br>
 *	異なるメモリシステムで解放等を行うとメモリが破壊される恐れがありますので注意が必要です。<br>
 *
 *	@ingroup	grp_Memory
 */


namespace OrgSys {
	namespace Memory {

		class IMemorySystem;

	/*!@addtogroup grp_MemorySystem */
	/*!@{*/

		const u16 cBlockMemoryUnit = 16;		//!< アライメント及びサイズの最小単位
		const u8  cAllocFillCodeValue=0xcd;		//!< alloc時のFILL時の値(デバッグ用)
		const u8  cFreeFillCodeValue=0xfd;		//!< free時のFILL時の値(デバッグ用)

		/*!
		 *	@brief メモリシステム共通インターフェイス
		 *
		 *	IMemorySystem は全ての種類のメモリシステムで共通する機能を提供するインターフェースクラスです。<br>
		 *	IMemorySystem でプログラムを記述する事でのちにメモリシステムを変更する事になった場合でも
		 *	最低限の修正で対応できます。
		 */
		class IMemorySystem {
		protected:

			/*!
			 *	@internal
			 *	@brief コンストラクタ
			 */
			IMemorySystem() {
				instance = this;
			}

			/*!
			 *	@internal
			 *	@brief 仮想デストラクタ
			 */
			virtual ~IMemorySystem() {
			}

			/*!
			 *	@brief インスタンスの取得
			 *	@return メモリシステムのインスタンス
			 *
			 *	@note
			 *	メモリシステムのインスタンスを取得します。
			 */
			static IMemorySystem *getInst() {
				return instance;
			}

			/*!
			 *	@brief 初期化
			 *	@param[in] heapAddr    ヒープアドレス
			 *	@param[in] heapSize    ヒープサイズ
			 *
			 *	@note
			 *	ヒープメモリの先頭アドレスとサイズを渡してメモリシステムを初期化します。
			 */
			virtual void initialize( void * heapAddr, size_t heapSize) = 0;

			/*!
			 *	@brief メモリ確保
			 *	@param[in] size 要求サイズ
			 *	@param[in] align アライメント
			 *	@return 確保されたアドレス
			 *
			 *	@note
			 *	管理しているヒープ領域から要求されたサイズのメモリブロックを確保して、<br>
			 *	そのアドレスを返り値として返します。メモリ領域が足りない場合、NULLを返します。<br>
			 *	アライメントが指定されている場合はその倍数になるようアドレスが調整されます。
			 */
			virtual void* mallocAlignment( size_t size, unsigned int align=1 ) = 0;

			/*!
			 *	@brief 解放
			 *	@param[in] ptr 解放したいメモリブロックアドレス
			 *
			 *	@note
			 *	mallocAlignment で確保したメモリを解放します。
			 *	異なるメモリシステムから確保されたメモリを解放した時の動作は保障されません。
			 */
			virtual void free( void * ptr ) = 0;

			/*!
			 *	@brief 空きメモリの総サイズ取得
			 *	@return 空きメモリの総サイズ
			 *
			 *	@note
			 *	メモリシステム内の未使用領域の合計 byte数を返します。<br>
			 *	確保できる最大サイズを取得するには getAllocatableSize を使用して下さい。
			 *
			 *	@sa getAllocatableSize
			 */
			virtual size_t getFreeTotalSize() const = 0;

			/*!
			 *	@brief 最大空きメモリブロックのサイズ取得
			 *	@param[in] align アライメント
			 *	@return 確保可能な最大
			 *
			 *	@note
			 *	このメモリシステムで確保可能な最大サイズを返します。
			 *
			 *	@sa getFreeTotalSize
			 */
			virtual size_t getAllocatableSize( unsigned int align ) const = 0;

			/*!
			 *	@brief メモリリークを調査する(デバッグ用)
			 *	@retval		false	メモリリークは発生していない
			 *	@retval		true	メモリリークは発生している(かもしれない)
			 *
			 *	@note
			 *	本メソッドを実行する前に予め、startMemoryLeak を実行しておく必要があります。
			 */
			virtual bool checkMemoryLeak() = 0;

			IMemorySystem		*instance;		//!< インスタンス

		};
	/*!@}*/
	}
}
